%Advanced Data Analysis and Visualization System
%Includes multiple visualization methods, statistical analysis, spectrogram 
%visualization, FFT spectrum analysis, with automatic and real-time data 
%processing, interactive controls, custom filtering options, and data 
%importing/exporting capabilities.
%Written by Chad Jordan
%6/18/2009
function Plots
clear variables; close all; clc
clear global

%Create main figure and GUI elements
fig = figure('Name', 'Advanced Analysis System', ...
    'Units', 'normalized', 'Position', [0 0 1 1], ...
    'MenuBar', 'none', 'NumberTitle', 'off');

%Generate rich dataset
t = linspace(0, 10, 1000);
signal1 = 2*sin(2*pi*0.5*t);
signal2 = cos(2*pi*2*t);
signal3 = 0.5*sin(2*pi*4*t);
composite = signal1 + signal2 + signal3;

%Initialize data structure
data.raw = composite';
data.processed = data.raw;
data.t = t;
data.signal1 = signal1;
data.signal2 = signal2;
data.signal3 = signal3;
data.fs = length(t)/max(t);
setappdata(fig, 'DataStore', data);

%Create control panel
control_panel = uipanel('Parent', fig, ...
    'Position', [0.01 0.01 0.15 0.98]);

%Data Import Button
uicontrol(control_panel, 'Style', 'pushbutton', ...
    'String', 'Import Data', ...
    'Position', [10 650 120 30], ...
    'Callback', @import_data);

%Filter Selection
uicontrol(control_panel, 'Style', 'popup', ...
    'String', {'No Filter|Lowpass|Highpass|Bandpass|Moving Average'}, ...
    'Position', [10 600 120 30], ...
    'Callback', @apply_filter);

%Export Options
uicontrol(control_panel, 'Style', 'pushbutton', ...
    'String', 'Export Results', ...
    'Position', [10 550 120 30], ...
    'Callback', @export_results);

%Real-time Toggle
uicontrol(control_panel, 'Style', 'togglebutton', ...
    'String', 'Real-time Mode', ...
    'Position', [10 500 120 30], ...
    'Callback', @toggle_realtime);

%Analysis Options
uicontrol(control_panel, 'Style', 'listbox', ...
    'String', {'Basic Stats', 'Advanced Stats', 'FFT', 'Wavelet', 'PSD'}, ...
    'Position', [10 350 120 100], ...
    'Callback', @update_analysis);

%Create plot areas
plot_panel = uipanel('Parent', fig, ...
    'Position', [0.17 0.01 0.82 0.98], ...
    'BackgroundColor', 'white', ...
    'Title', 'Analysis Display');

%Use plot_panel as parent for all subplot elements
subplot(3,3,1, 'Parent', plot_panel)
subplot(3,3,2, 'Parent', plot_panel)
subplot(3,3,3, 'Parent', plot_panel)

%Callback functions
function import_data(~, ~)
    [filename, pathname] = uigetfile({'*.txt;*.csv;*.dat', 'Data Files'});
    if filename ~= 0
        data = getappdata(fig, 'DataStore');
        data.raw = load(fullfile(pathname, filename));
        data.processed = data.raw;
        setappdata(fig, 'DataStore', data);
        update_plots;
    end
end

function apply_filter(hObject, ~)
    filter_type = get(hObject, 'Value');
    data = getappdata(fig, 'DataStore');
    
    %Input for Lowpass, Highpass, Bandpass, and Moving Average
    switch filter_type
        case 2
            [b,a] = butter(4, 0.1);
            data.processed = filtfilt(b, a, data.raw);
        case 3
            [b,a] = butter(4, 0.5, 'high');
            data.processed = filtfilt(b, a, data.raw);
        case 4
            [b,a] = butter(4, [0.1 0.5]);
            data.processed = filtfilt(b, a, data.raw);
        case 5
            data.processed = smooth(data.raw, 10);
    end
    
    setappdata(fig, 'DataStore', data);
    update_plots;
end

function export_results(~, ~)
    data = getappdata(fig, 'DataStore');
    [filename, pathname] = uiputfile({'*.mat', 'MAT-files (*.mat)'});
    if filename ~= 0
        results.raw = data.raw;
        results.processed = data.processed;
        results.stats = calculate_statistics(data.processed);
        save(fullfile(pathname, filename), 'results');
        
        %Export figures
        saveas(gcf, [pathname 'analysis_plots.fig']);
        print('-dpng', '-r300', [pathname 'analysis_plots.png']);
    end
end

function toggle_realtime(hObject, ~)
    if get(hObject, 'Value')
        start_realtime_processing;
    else
        stop_realtime_processing;
    end
end

function update_analysis(hObject, ~)
    selected = get(hObject, 'Value');
    data = getappdata(fig, 'DataStore');
    
    switch selected
        case 1 
            stats = calculate_statistics(data.processed);
            display_stats(stats);
        case 2 
            advanced_statistical_analysis(data.processed);
        case 3 
            perform_fft_analysis(data.processed, data.fs);
        case 4 
            perform_wavelet_analysis(data.processed);
        case 5 
            calculate_power_spectral_density(data.processed, data.fs);
    end
end

    function update_plots
    data = getappdata(fig, 'DataStore');
    
    if ~isempty(data.raw)
        %Plot 1: Time Domain Signals
        subplot(3,3,1)
        plot(data.t, data.signal1, 'b', data.t, data.signal2, 'r', data.t, data.signal3, 'g', 'LineWidth', 1.5)
        title('Individual Components')
        xlabel('Time (s)')
        ylabel('Amplitude')
        legend('0.5 Hz', '2 Hz', '4 Hz')
        grid on
        
        %Plot 2: Raw Data Analysis
        subplot(3,3,2)
        plot(data.raw)
        title('Raw Data')
        grid on
        
        %Plot 3: Processed Signal
        subplot(3,3,3)
        plot(data.processed)
        title('Processed Data')
        grid on
        
        %Plot 4: FFT Analysis
        subplot(3,3,4)
        perform_fft_analysis(data.processed, data.fs)
        
        %Plot 5: Spectrogram
        subplot(3,3,5)
        spectrogram(data.processed, 256, 250, 256, data.fs, 'yaxis')
        title('Spectrogram')
        colorbar
        
        %Plot 6: Statistics
        subplot(3,3,6)
        stats = calculate_statistics(data.processed);
        bar(cell2mat(struct2cell(stats)))
        title('Statistical Analysis')
        
        %Plot 7: 3D Visualization
        subplot(3,3,[7,8,9])
        [X,Y] = meshgrid(linspace(-2,2,50), linspace(-2,2,50));
        Z = sin(sqrt(X.^2 + Y.^2)).*exp(-0.2*(X.^2 + Y.^2));
        surf(X,Y,Z)
        title('3D Surface with Dynamic Properties')
        xlabel('X-axis')
        ylabel('Y-axis')
        zlabel('Amplitude')
        colormap jet
        shading interp
        lighting phong
        camlight
        rotate3d on
        view(-37.5, 30)
        colorbar
        
        %Add statistics box
        mean_val = mean(data.processed);
        max_val = max(data.processed);
        min_val = min(data.processed);
        std_val = std(data.processed);
        dim = [.15 .02 .3 .05];
        str = sprintf('Mean: %.3f | Max: %.3f | Min: %.3f | STD: %.3f', ...
            mean_val, max_val, min_val, std_val);
        annotation('textbox', dim, 'String', str, 'FitBoxToText', 'on', ...
            'BackgroundColor', 'white');
    end
end

%Initialize the system
update_plots;
end

%Helper functions (defined outside main function)
function stats = calculate_statistics(data)
    stats.mean = mean(data);
    stats.std = std(data);
    stats.max = max(data);
    stats.min = min(data);
    stats.kurtosis = kurtosis(data);
    stats.skewness = skewness(data);
    stats.rms = rms(data);
end

function perform_fft_analysis(data, fs)
    N = length(data);
    Y = fft(data);
    P2 = abs(Y/N);
    P1 = P2(1:N/2+1);
    P1(2:end-1) = 2*P1(2:end-1);
    f = fs*(0:(N/2))/N;
    plot(f, P1)
    title('FFT Analysis')
    xlabel('Frequency (Hz)')
    ylabel('Magnitude')
    grid on
end